from PyQt5.QtGui import QIcon, QDesktopServices
from PyQt5.QtXml import QDomDocument
from qgis.core import QgsComposition, QgsProject, QgsMapLayerStyle
from PyQt5.QtWidgets import QAction, QApplication
import tempfile
import os.path
from PyQt5.QtCore import QUrl, Qt, QDate
from ..b64_conv import B64Convertor
import math

class ComposerValidation(object):
    """Class to open a composer then validate its export"""

    def __init__(
        self, feature, template_path, iface, substitution_map=dict(),
        zoom_to_feature=False, load_feature_in_sub_map=False, callback=None,
        zoom_to_feature_scale=1.1, scale_step=None, output_pdf=False,
        output_png=False, layers_config=dict(), output_filename=None,
    ):
        self.feature = feature
        self.template_path = template_path
        self.iface = iface
        self.substitution_map = substitution_map
        self.zoom_to_feature = zoom_to_feature # list of mapcomposeItem name # TODO
        self.zoom_to_feature_scale = zoom_to_feature_scale
        self.load_feature_in_sub_map = load_feature_in_sub_map
        self.callback = callback
        self.output_png = output_png
        self.output_pdf = output_pdf
        self.scale_step = scale_step
        self.output_filename = output_filename

        self.composer_map = dict()

        # a dict (indexed form name of MapComposerItem) containing
        # list of layers + path to a style file
        # ie [(l, 'xx.qml'), (l2, None)]
        self.layers_config = layers_config

    def export_img_path(self, ext):
        """
        ext: pdf ou png
        """
        output_filename = 'export'

        if self.output_filename:
            output_filename = self.output_filename

        return os.path.join(tempfile.gettempdir(), "{}.{}".format(output_filename, ext))

    def export_png(self):
        img = self.composition.printPageAsRaster(0)
        img_path = self.export_img_path('png')
        img.save(img_path)
        return img_path

    def export_pdf(self):
        out_path = self.export_img_path('pdf')
        self.composition.exportAsPDF(out_path)
        return out_path

    def add_substitution(self, key, value):
        """
        Add an element to the substitution_map
        """
        self.substitution_map[key] = str(value)

    def add_substitutions(self, dico):
        """
        Add elements to the substitution_map
        """
        for key in dico:
            self.add_substitution(key, dico[key])


    def get_substitution_map(self):
        """
        Retourne un dict d tel que pour toute clef key, tout texte [key] sera
        remplace par d[key]

        Exemple si d = {'nom':'marc'}, alors le texte "Bonjour [nom]" sera
        remplace par "Bonjour marc".
        """
        return self.substitution_map

    def load_feature_in_sub_map_fct(self):
        for field in self.feature.fields():
            field_name = field.name()
            field_data = self.feature[field_name]

            if isinstance(field_data, QDate):
                self.substitution_map[field_name] = field_data.toString("dd.MM.yyyy")
            else:
                self.substitution_map[field_name] = str(field_data)

    def generate_composer(self):
        template = QDomDocument()
        with open(self.template_path, encoding="utf-8") as template_file:
            template.setContent(template_file.read())
        template_file.close()

        if self.load_feature_in_sub_map:
            self.load_feature_in_sub_map_fct()

        self.composition = QgsComposition(QgsProject.instance())
        self.composition.loadFromTemplate(
            template, self.get_substitution_map())

        for cm in self.composition.composerMapItems():
            self.composer_map[cm.displayName()] = cm

    def exec_(self):
        """
        Affichage du composer puis quand valider creation de l'image
        """
        self.generate_composer()

        self.composition.refreshItems()
        self.composer_iface = self.iface.openComposer(self.composition)

        if self.zoom_to_feature:
            composer_map_item = self.composition.composerMapItems()[-1]  # FIXME only one map

            bb = self.feature.geometry().boundingBox()
            bb.scale(self.zoom_to_feature_scale)
            composer_map_item.zoomToExtent(bb)

            if self.scale_step:
                scale = composer_map_item.scale()
                rounded_scale = math.ceil(scale / self.scale_step)\
                    * self.scale_step
                composer_map_item.setNewScale(rounded_scale)


        for composer_map_name in self.layers_config:
            composer_map = self.composer_map[composer_map_name]
            layers_config = self.layers_config[composer_map_name]

            if layers_config:
                composer_map.setLayers([l for l, _ in layers_config])

                layer_style_overrides = composer_map.layerStyleOverrides()

                for layer, style_file_path in layers_config:
                    if layer and style_file_path:
                        with open(style_file_path, encoding="utf-8") as style_file:
                            style = QgsMapLayerStyle(style_file.read())
                            sm = layer.styleManager()
                            sm.removeStyle('compo ' + composer_map_name)
                            sm.addStyle('compo ' + composer_map_name, style)
                            layer_style_overrides[layer.id()] = 'compo ' + composer_map_name
                        style_file.close()

                composer_map.setLayerStyleOverrides(layer_style_overrides)
                composer_map.setKeepLayerSet(True)  # force to use the layerSet
                composer_map.setKeepLayerStyles(True)  # force to use the style

        self.check_composer_window = self.composer_iface.parent()
        self.check_composer_window.destroyed.connect(self.close_callback)

        toolbar = self.check_composer_window.addToolBar('sif-td')
        icon = QIcon(os.path.join(os.path.dirname(__file__), "..", "icons", "icon_validate.png"))
        action = QAction(
            icon, 'Valider le composer', self.check_composer_window)
        action.triggered.connect(self.exec_callback)
        action.setEnabled(True)
        toolbar.addAction(action)

    def close_callback(self, event):
        # to execute when the dialog is closed
        pass

    def get_callback_to_store_into_feature(self, layer, feature,
            field_pdf_name, field_png_name):
        def callback(pv_pdf_path, pv_png_path):
            layer.startEditing()

            if field_pdf_name:
                b64data_pdf = B64Convertor.base64_from_file(pv_pdf_path)
                layer.changeAttributeValue(
                    feature.id(),
                    layer.fields().indexFromName(field_pdf_name), b64data_pdf)

            if field_png_name:
                b64data_png = B64Convertor.base64_from_file(pv_png_path)
                layer.changeAttributeValue(
                    feature.id(),
                    layer.fields().indexFromName(field_png_name), b64data_png)

        return callback


    def exec_callback(self):
        """
        Callback quand user clique sur bouton
        """
        QApplication.setOverrideCursor(Qt.WaitCursor)

        output_png_path = None
        if self.output_png:
            output_png_path = self.export_png()

        output_pdf_path = None
        if self.output_pdf:
            output_pdf_path = self.export_pdf()

        QApplication.restoreOverrideCursor()

        self.check_composer_window.close()

        if self.callback:
            self.callback(output_pdf_path, output_png_path)
        else:
            if output_pdf_path:
                # Default behavior
                QDesktopServices.openUrl(
                    QUrl.fromLocalFile(output_pdf_path))
            elif output_png_path:
                # Default behavior
                QDesktopServices.openUrl(
                    QUrl.fromLocalFile(output_png_path))
