from qgis.core import QgsProject, QgsMessageLog, QgsMapLayer
from PyQt5.QtCore import QByteArray
from PyQt5.QtWidgets import QToolBar
import os

from .paggf_utils import PAGGFUtils, update_background_layer
from .paggf_layers import LoadedLayers


class PAGGFLoader(object):
    """
    Load a version of the PAGGF plugin in function of the project
    which is opened
    """

    def __init__(self, iface):
        self.iface = iface
        self.pjt = QgsProject.instance()
        self.pjt.readProject.connect(
            self.load_new_version_at_pjt_reading)  # when a new project is
        # loaded -> try to load the plugins
        self.pjt.removeAll.connect(
            self.unload_running_version)  # when changing project we unload
        # the plugin
        self.running_version = None  # Version of the plugin which is running

    def initGui(self):
        # At start check if a project is already opened
        self.load_new_version_at_pjt_reading()

    def unload_running_version(self):
        if self.running_version:
            self.running_version.unload()
            # del self.running_version
            self.running_version = None

    def unload(self):
        self.unload_running_version()

    def load_new_version_at_pjt_reading(self):
        """
        When a new project is loaded, load the version of the plugin relative
        to this project
        """
        self.unload_running_version()

        if os.path.split(self.pjt.fileName())[1] == 'paggf_encodage_bm.qgs':
            QgsMessageLog.logMessage('Load BM', 'PAGGF', QgsMessageLog.ALL)
            from .paggf_version_bm import PAGGF_BM
            self.running_version = PAGGF_BM(self.iface)
            self.running_version.initGui()
        elif os.path.split(self.pjt.fileName())[1] == 'paggf_encodage_cid.qgs':
            QgsMessageLog.logMessage('Load CID', 'PAGGF', QgsMessageLog.ALL)
            from .paggf_version_cid import PAGGF_CID
            self.running_version = PAGGF_CID(self.iface)
            self.running_version.initGui()
        elif os.path.split(self.pjt.fileName())[1] == 'paggf_encodage_ef.qgs':
            QgsMessageLog.logMessage('Load EF', 'PAGGF', QgsMessageLog.ALL)
            from .paggf_version_ef import PAGGF_EF
            self.running_version = PAGGF_EF(self.iface)
            self.running_version.initGui()
        else:
            QgsMessageLog.logMessage('Do nothing', 'PAGGF', QgsMessageLog.ALL)
            return

        layers = LoadedLayers()
        utils = PAGGFUtils(layers)

        # Background layer
        update_background_layer()

        # Load mask layer
        utils.mask_layer.create_or_update()

        # Toolbar positions
        if self.running_version:
            # Restore toolbar positions
            if hasattr(self.running_version, "TOOLBAR_POSITIONS"):
                self.iface.mainWindow().restoreState(QByteArray(self.running_version.TOOLBAR_POSITIONS))

            # Set the active layer
            self.iface.setActiveLayer(self.running_version.default_active_layer())

        # FIXME: waiting for the "trust" option to be merged in qgis
        for ln, lyr in QgsProject.instance().mapLayers().items():
            if lyr.type() == QgsMapLayer.VectorLayer:
                lyr.updateExtents()


        # enable the digitize toolbar only for selected layers
        digitize_tb = self.iface.mainWindow().findChild(QToolBar, "mDigitizeToolBar")
        editable_layers = [layers.get("gis", l) for l in ("parcel_rivers_to_valid", "parcel_buildings_to_valid", "parcel_roads_to_valid")]
        def on_layer_changed(layer):
            is_allowed = layer is not None and layer in editable_layers
            digitize_tb.setVisible(is_allowed)
        self.iface.layerTreeView().currentLayerChanged.connect(on_layer_changed)
        digitize_tb.setVisible(False)
