from PyQt5 import uic
from PyQt5.QtWidgets import QDialog, QMessageBox
import os.path
from ..paggf_layers import LoadedLayers, cid_translator_table
from ..paggf_utils import PAGGFUtils
from ..composer.composer_utils import load_from_template
import tempfile
from qgis.core import QgsComposerHtml, QgsComposerFrame
from PyQt5.QtGui import QPageSize, QDesktopServices
from PyQt5.QtCore import QUrl, QDate
from PyQt5.QtGui import QIcon

FORM_CLASS, _ = uic.loadUiType(os.path.join(
    os.path.dirname(__file__), 'CID_munip_list.ui'))


class CIDMunipListDlg(QDialog, FORM_CLASS):
    def __init__(self, municipality_id, parent=None):
        super(CIDMunipListDlg, self).__init__(parent)
        self.setupUi(self)
        self.layers = LoadedLayers()
        self.utils = PAGGFUtils(self.layers)
        self.municipality_id = municipality_id
        self.parcel_layer = self.layers.get('gis', 'current_idc_parcels')
        self.fields_aliases_dict = self.parcel_layer.attributeAliases()
        self.fields_aliases_dict_inv = {
            v: k for k, v in self.fields_aliases_dict.items()}

        self.add_button.setIcon(QIcon(os.path.join(
            os.path.dirname(__file__), "..", "icons", "symbologyAdd.svg")))
        self.add_button.setText("Ajouter")
        self.remove_button.setIcon(QIcon(os.path.join(
            os.path.dirname(__file__), "..", "icons", "symbologyRemove.svg")))
        self.remove_button.setText("Supprimer")

        self.add_button.clicked.connect(self.move_item(self.fields_for_selection, self.selected_fields))
        self.remove_button.clicked.connect(self.move_item(self.selected_fields, self.fields_for_selection))


        self.default_selection_name = [
            'end_date', 'idc_parcel_code', 'hill_district_name',
            'sub_hill_avenue_name', 'usage', 'conflicting_parts', 'area', 'adjacent_list'
        ]

        self.do_not_propose_name_for_selection = [
            'b_id', 'id_polygon', 'id_split_origin_parcel', 'id_inventory',
            'is_polygon_valid', 'polygon_invalidity', 'id', 'idc_parcel_code',
            "doc_pdf_parcel_sketch",
            "doc_png_parcel_sketch",
            "doc_generated_report",
            "doc_signed_report",
            "doc_idc_report_signatures",
            "doc_mediation_report",
            "doc_conflict_report",
            "manager", "manager_other",
            "status_validated_by_operator", "status_validated_by_expert",
            "status_validated_by_president", "status_report_signed",
            "validity_event",
            "commission_reference"
        ]

        for f in self.parcel_layer.dataProvider().fields():
            f_name = f.name()
            if f_name not in self.default_selection_name and\
                    f_name not in self.do_not_propose_name_for_selection:
                self.fields_for_selection.addItem(
                    self.alias_for_feature(f))

        for n in self.default_selection_name:
            self.selected_fields.addItem(self.alias_or_name(n))

        self.buttonBox.accepted.connect(self.accepted_slot)

    def exec_(self):
        self.municipality = self.utils.get_municipality(self.municipality_id)

        if not self.municipality:
            msg = QMessageBox()
            msg.setText("L'outil est mal configuré : la commune dont \
l'identifiant est {} n'existe pas dans la base de donnée".format(
                self.municipality_id))
            msg.exec_()
        else:
            super(CIDMunipListDlg, self).exec_()

    def move_item(self, list_from, list_to):
        def ret():
            s_item = list_from.selectedItems()

            for item in s_item:
                item_row = list_from.row(item)
                list_from.takeItem(item_row)
                list_to.addItem(item.text())
        return ret


    def alias_for_feature(self, f):
        """
        Get the alias for a field. If the alias does not exists the function
        return the name of the field.
        """
        return self.alias_or_name(f.name())

    def alias_or_name(self, name):
        """
        Given a name return the alias or the name itseft if if does not exists
        """
        if name in self.fields_aliases_dict:
            return self.fields_aliases_dict[name]
        return name

    def name_from_alias(self, alias):
        """
        Get the field name from the alias.
        """
        if alias in self.fields_aliases_dict_inv:
            return self.fields_aliases_dict_inv[alias]
        else:  # pas eu de conversion par fct alias_or_name, donc alias = name
            return alias

    def accepted_slot(self):
        seleted_fields_alias = []

        for i in range(len(self.selected_fields)):
            seleted_fields_alias.append(self.selected_fields.item(i).text())

        selected_field_names = [
            self.name_from_alias(a) for a in seleted_fields_alias]


        province = self.utils.get_province_by_munip_id(self.municipality_id)

        html_content = "<table><thead><tr>"

        html_content += "<th>N°</th>"

        for alias in seleted_fields_alias:
            html_content += "<th>{}</th>".format(alias)

        html_content += "</tr></thead>"
        html_content += "<tbody>"

        it = 1

        for p in self.utils.get_encoded_idc_parcels_for(self.municipality):
            html_content += "<tr>"

            html_content += "<td class=\"id\">{}</td>".format(p["list_id"])

            for field_name in selected_field_names:
                field_data = p[field_name]
                if isinstance(field_data, QDate):
                    str_data = field_data.toString("dd.MM.yyyy")
                elif isinstance(field_data, bool):
                    str_data = 'Oui' if field_data else 'Non'
                elif field_name in cid_translator_table:
                    str_data = self.layers.translate_en_to_fr(
                        'tr', cid_translator_table[field_name], field_data)
                else:
                    str_data = str(field_data)

                html_content += "<td>{}</td>".format(str_data)

            html_content += "</tr>"

            it += 1

        html_content += "</tbody>"
        html_content += "</table>"

        composition = load_from_template(
            'CID_A3_municipality_list.qpt',
            substitution_map={
                'LOGO_BURUNDI_PATH': os.path.join(self.utils.plugin_dir, 'img/Blason_du_Burundi.svg'),
                'province': province['name'],
                'municipality': self.municipality['name']}
        )

        html_item = QgsComposerHtml(composition, False)

        paper_size = QPageSize.size(QPageSize.A3, QPageSize.Millimeter)
        margin_bottom = 10
        margin_top = 70
        margin_left = 10
        paper_width = paper_size.height()  # landscape mode
        paper_height = paper_size.width()  # landscape mode

        html_frame = QgsComposerFrame(
            composition,
            html_item,
            margin_left,
            margin_top,
            paper_width - 2 * margin_left,
            paper_height - margin_top - margin_bottom)

        html_frame.setItemPosition(
            margin_left,
            margin_top,
            page=1
        )

        html_item.addFrame(html_frame)

        html_item.setResizeMode(QgsComposerHtml.ExtendToNextPage)

        html_item.setContentMode(QgsComposerHtml.ManualHtml)

        html_item.setUserStylesheet("\
table {\
    border-collapse: collapse;\
    empty-cells: show;\
    width: 100%;\
    font-family: 'Arial', 'Liberation Sans', 'Noto Sans';\
}\
\
table, th, td {\
    border: 1px solid black;\
}\
table td.id {\
	background-color: #BBB;\
	color: #008000;\
	text-align: center\
}\
")

        html_item.setUserStylesheetEnabled(True)
        html_item.setHtml(html_content)
        html_item.loadHtml()

        out_path = os.path.join(
            tempfile.gettempdir(), "list_munip.pdf")

        composition.exportAsPDF(out_path)

        QDesktopServices.openUrl(QUrl.fromLocalFile(out_path))
